'This program estimates the cumulative distribution function
'         (critical values) of the unit root test statistic (t-ratio of lambda)
'         for a time series model with asymptotically decaying fractional trend, 
'         f(t) = gamma/(1 + delata*t), across simulated random walks with
'         innovations ~ N(0, 1).
'         (The Dickey-Fuller test statistics for tests with no constant, with
'        constant, and with trend can be estimated as well.) 

'         See ADtrend-Instructions.pdf for details.
'         Version: 2016.02

'         Author: Konstantin Gluschenko
'                       glu@nsu.ru
'                       http://nsu-ef.ru/teachers/Gluschenko_KP/

'ARGUMENTS of the program may follow in any order. The only optional argument is DF. If the workfile already exists on disc (when an estimation is continued after some break), only the filename should be provided (then keyword "file=" may be omitted).

'The arguments are as follows: 
'(<...> means some value of argument; [...] means optionalality; | means a choice, i.e. "or"):
' file=[<path>]<file_name> is the workfile name 
'          (if there are blanks, enclose it in quotes).
' T=<number> is the sample size. 
' N=<number> is the number of  replications.
' [DF=yes|no] is the mode of estimating Dickey-Fuller distributions;
'           by default, DF=no (implying no estimating).         

'EXAMPLES: 
'  file="test 1" DF=no N=10000 T=100
'  file=50 N=100000 T=50
'  T=78 file="d:\projects\Nonlinear trend\nlt78.wf1" N=500000 DF=yes 

' Before starting the program, set "Maximum errors before halting"
'    to a number about N/100 or more ! 

' If the workfile specified already exists on disk, the program cannot 
'    change its parameters T, N, and DF.

'RESULTS:
'CDF_t_nlt is CDF of the t-ratio of the autoregressive coefficient (minus one) in the equation with nonlinear, asymptotically decaying trend. If estimation of the Dickey-Fuller CDFs is prescribed (DF=yes), CDF_DF, CDF_DF_c, and CDF_DF_t are CDFs for the case of no constant, constant, and trend, respectively. The first column in these matrices is probability (from 0 to 1 with step 0.001); the second column is critical values.
'Matrix t_nlt (and, if exist, t_DF, t_DF_c, and t_DF_t) contains the full set of estimates of the t-ratio of lambda.
'Table DESCRIPTION reports conditions of the given estimation. 

'*************************** SUBROUTINES ***************************

'-------------- Transform seconds to hours and minutes ------------------
Subroutine Seconds_to_HM(scalar Total_time, string %Time)
     !minutes=@round(Total_time/60)
     !hours=@floor(!minutes/60)
     !minutes=!minutes-!hours*60
     %Time=""
          if !hours=1 then
               %Time="1 hour "
          endif
          if !hours>1 then
               %Time=@str(!hours)+" hours "
          endif
    %Time=%Time+@str(!minutes)+" min. "
EndSub 'Seconds_to_HM

'------------ Computing CDFs of a given set of t-ratio series -------------
Subroutine CDF(vector Tau, matrix CDF, scalar Quantiles)
!Rows=@rows(Tau)
Tau=@sort(Tau)
     for !i=1 to Quantiles
          CDF(!i,1)=(!i-1)/Quantiles
          !k=@floor((2*!i-1)*!Rows/(2*Quantiles))
               if !k=0 then
                    !k=1
                endif
           CDF(!i,2)=Tau(!k)
     next
CDF(Quantiles+1,1)=1
CDF(Quantiles+1,2)=Tau(!Rows)
EndSub 'CDF

'****************** PREPARATORY PROCEDURES ******************
tic        'Reset timer

'------- READ PROGRAM ARGUMENTS AND FIND WORKFILE NAME -------
%Source=%0+" "+%1+" "+%2+" "+%3
     if %4<>"" then
          StatusLine Superfluous argument(s): ... {%4} {%5} {%6} {%7}
          STOP
     endif
%file=""
%T=""
%N=""
%DF="no"
     for %argument 0 1 2 3
          %arg=%{%argument}
          !L=@Length(%arg)
               if %1<>"" and @left(%arg,5)<>"file=" and @left(%arg,2)<>"T=" and @left(%arg,2)<>"N=" and @left(%arg,3)<>"DF=" then
                    StatusLine  Unrecognized argument: {%arg} 
                    STOP  
               endif
               if @left(%arg,5)="file=" then  
                    %file=@right(%arg,!L-5)
               endif
               if @left(%arg,2)="T=" then  
                    %T=@right(%arg,!L-2)
               endif
               if @left(%arg,2)="N=" then  
                    %N=@right(%arg,!L-2)
               endif
               if @left(%arg,3)="DF=" then  
                         if %arg="DF=yes" then
                              %DF="yes"
                         else
                              %DF=%arg
                         endif
               endif
     next 
     if %1="" and %file="" then       'only filename is specified
          %file=%0     
     endif
     if %file="" then
          StatusLine No workfile name specified: {%Source}
          STOP
     endif
!errors=@errorcount
logmode -errors
Load {%file}                      'Try to load workfile
     If %1="" and !errors<>@errorcount then
          logmode errors
          StatusLine Only filename is specified, implying the continuation of interupted work. However, file %file is not found on disc.
          STOP 
     endif
logclear
logmode errors l

'---------------------- WORKFILE %0 EXISTS ON DISK. ----------------------
'                                THEN READ ITS PARAMETERS

     if !errors=@errorcount then 'old file
                if %1<>"" then
                    StatusLine The file already exists on disc. Then other arguments are superfluous: {%Source}
                    STOP                    
               endif
               if Description(5,2)<>"" then
                    SetCell(Description,5,2,@str(Current),"r")
               else
                    StatusLine The estimation has been completed.
                    STOP
               endif
          !T_=T                                           'Sample size
          !N=N                                            'Number of replications
               if DF=1 then
                    %DF="yes"
               endif
     else 'new file 

'---------------------- WORKFILE %0 DOES NOT EXIST. ---------------------
'                        THEN CHECK PROGRAM ARGUMENTS        
'                             AND CREATE NEW WORKFILE %0

'***** Check arguments
                    if %T="" then
                         StatusLine No sample size specified: {%Source}
                         STOP
                    endif 
           !T_=@val(%T)
                    if !T_=NA or !T_<5 or @round(!T_)<>!T_ then 
                         StatusLine Invalid samle size: T={%T}
                         STOP
                    endif
                    if %N="" then
                         StatusLine No number of replications specified: {%Source}
                         stop
                    endif 
           !N=@val(%N)
                    if !N=NA or !N<1 or @round(!N)<>!N then 
                         StatusLine Invalid number of replications: N={%N}
                         stop
                    endif
                    if %DF="no" then
                         %Source=%Source + "    [DF=no by default]"
                    endif
                    if %DF<>"no" and %DF<>"yes" and %DF<>"DF=no" then
                         %Source=%Source + "    [Invalid argument DF. Interpreted as DF=no]"                  
                    endif  
          
'***** Create new workfile
           WorkFile {%file} u 1 !T_
          Scalar T=!T_
          Scalar N=!N
          Scalar Seconds=0
          Scalar Current=1
          Scalar DF=0
          'Estimated t-stats: 
          Vector(N) t_nlt=NA
               if %DF="yes" then
                    Vector(N) t_DF_c=NA
                    Vector(N) t_DF_t=NA
                    Vector(N) t_DF_t2=NA
                    DF=1
               endif
   
          Table(11,9) Description
          Description(1,1)="CDF of UNIT ROOT TEST STATISTICS FOR EQUATION WITH FRACTIONAL TREND"
          Description(2,1)="h(t) = gamma/(1 + delata*t)"
          Description(3,1)="Sample size: "+%T
          Description(4,1)="Number of replications: "+%N
          SetCell(Description,5,2,"0","r")
          Description(5,3)="or more performed (see scalar CURRENT for exact figure) "
          Description(6,1)="Started "+@date+" at "+@time
          Description(9,1)=%Source
     endif  'new file

'************************ ESTIMATE MODEL *************************
Series Y=0
Series eps
Series res
Series time=@trend
Series time_sq=time*time
!m=5                                 'number  of initial approximations
vector(!m) SSR
vector(!m) tau
Equation ARo                 'Dickey-Fuller test equation with no constant
Equation ARc                 'Dickey-Fuller test equation with constant
Equation ARt                  'Dickey-Fuller test equation with trend
Equation ARt2                'Dickey-Fuller test equation with quadratic trend
Equation NL0                 'Equations with nonlinear trend
Equation NLo
Equation NLc
Equation NLt
Equation NLd
Equation Detrend            'Auxiliary equations
Equation ARr

Show Description
!Backup_time=1800 'seconds, i.e. 0.5 hour
!Last_backup=@toc
!Elapsed_time=Seconds+@toc
!k1=Current
%msg="Currently started: "+@date+" at "+@time
logmsg {%msg}

    for !k=!k1 to N
         Call Seconds_to_HM((Seconds/!k)*(N-!k),%a)
         StatusLine Replication {!k} of {!N}. Estimated time left {%a}
         Current=!k
         SSR=NA
         tau=NA
     'Generate a random walk
         eps=nrnd
         smpl @first+1 @last
         Y=Y(-1)+eps
         smpl @first @last

        'Initial approximation is 0
         !i=1
         c=0
         !errors=@errorcount
         NL0.ls(M=1000,C=1e-05) D(Y)=c(1)*Y(-1)+c(4)*(1/(1+c(5)*time)-(c(1)+1)/(1+c(5)*(time-1)))
         Freeze(Eq) NL0.results
              if !errors=@errorcount and NL0.@stderrs(1)<>NA and @left(Eq(6,1),24)<>"Convergence not achieved" then         
                   tau(!i)=NL0.@tstats(1)
                   SSR(!i)=NL0.@SSR
              endif
         Delete Eq

        'Initial approximation from coef of AR(1) with no constant
         !i=2
         c=0
         ARo.ls D(Y)=c(1)*Y(-1)                       'DF with no const
         !errors=@errorcount
         NLo.ls(M=1000,C=1e-05) D(Y)=c(1)*Y(-1)+c(4)*(1/(1+c(5)*time)-(c(1)+1)/(1+c(5)*(time-1)))
         Freeze(Eq) NLo.results
              if !errors=@errorcount and NLo.@stderrs(1)<>NA and @left(Eq(6,1),24)<>"Convergence not achieved" then         
                   tau(!i)=NLo.@tstats(1)
                   SSR(!i)=NLo.@SSR
              endif
         Delete Eq 

        'Initial approximation from coefs of AR(1) with const
         !i=3
         c=0
         ARc.ls D(Y)=c(1)*Y(-1)+c(2)                         'DF with const
              if abs(c(1))>1e-300 then
                   c(4)=-c(2)/c(1)
                  !errors=@errorcount
                  NLc.ls(M=1000,C=1e-05) D(Y)=c(1)*Y(-1)+c(4)*(1/(1+c(5)*time)-(c(1)+1)/(1+c(5)*(time-1)))
                  Freeze(Eq) NLc.results
                       if !errors=@errorcount and NLc.@stderrs(1)<>NA and @left(Eq(6,1),24)<>"Convergence not achieved" then         
                          tau(!i)=NLc.@tstats(1)
                           SSR(!i)=NLc.@SSR
                       endif
                  Delete Eq
              endif

        'Linear approximation: 1/(1+x) -> 1 - x
         !i=4
         c=0
         ARt.ls D(Y)=c(1)*Y(-1)+c(2)+c(3)*time     'DF with trend
         c(4)=(c(2)-c(3)*(c(1)+1)/c(1))/c(1)
         c(5)=-c(3)/(c(1)*c(4))
         !errors=@errorcount
         NLt.ls(M=1000,C=1e-05) D(Y)=c(1)*Y(-1)+c(4)*(1/(1+c(5)*time)-(c(1)+1)/(1+c(5)*(time-1)))
         Freeze(Eq) NLt.results
              if !errors=@errorcount and NLt.@stderrs(1)<>NA and @left(Eq(6,1),24)<>"Convergence not achieved" then         
                  tau(!i)=NLt.@tstats(1)
                  SSR(!i)=NLt.@SSR
              endif
         Delete Eq

        'Initial approximation is computed thru detrending
         !i=5
         c=0
         !errors=@errorcount
         Detrend.ls(M=1000,C=1e-05) Y=c(4)/(1+c(5)*time)
              if !errors=@errorcount then
                   res=resid
                   ARr.ls D(res)=c(1)*res(-1)
                   NLd.ls(M=1000,C=1e-05) D(Y)=c(1)*Y(-1)+c(4)*(1/(1+c(5)*time)-(c(1)+1)/(1+c(5)*(time-1)))
                   Freeze(Eq) NLd.results
                        if !errors=@errorcount and NLd.@stderrs(1)<>NA and @left(Eq(6,1),24)<>"Convergence not achieved" then         
                             tau(!i)=NLd.@tstats(1)
                             SSR(!i)=NLd.@SSR
                        endif
                   Delete Eq
              endif

          !minSSR=1e200
               for !i=1 to !m
                         if SSR(!i)<>NA and SSR(!i)<!minSSR then
                              !minSSR=SSR(!i)
                              t_nlt(!k,1)=tau(!i)
                         endif
               next
               if %DF="yes" then
                    t_DF_c(!k, 1)=ARc.@tstats(1)
                    t_DF_t(!k, 1)=ARt.@tstats(1)
                    ARt2.ls D(Y)=c(1)*Y(-1)+c(2)+c(3)*time+c(6)*time_sq 
                    t_DF_t2(!k, 1)=AR2t.@tstats(1)
               endif
         !toc=@toc
         Seconds=!Elapsed_time+!toc
               if !toc-!Last_backup>=!Backup_time then
                    SetCell(Description,5,2,@str(Current),"r")
                    Call Seconds_to_HM(Seconds,%a)
                    Description(8,1)="Elapsed time: "+%a
                    save {%file}
                    Show Description
                    !Last_backup=!toc
               endif
               if t_nlt(!k,1)=NA then
                    !k=!k-1
               endif
    next '!k

Current=Current+1
save {%file}

!q=1000 'the number of quantiles
matrix(!q+1,2) CDF_t_nlt
StatusLine Computing CDF
call CDF(t_nlt, CDF_t_nlt, !q)
     if %DF="yes" then
          StatusLine Computing Dickey-Fuller CDF 
          matrix(!q+1,2) CDF_DF_c
          call CDF(t_DF_c, CDF_DF_c, !q)
          matrix(!q+1,2) CDF_DF_t
          call CDF(t_DF_t, CDF_DF_t, !q)
          matrix(!q+1,2) CDF_DF_t2
          call CDF(t_DF_t2, CDF_DF_t2, !q)
     endif

Description(5,2)=""
Description(5,3)=""
Description(7,1)="Finished "+@date+" at "+@time
Seconds=!Elapsed_time+@toc
Call Seconds_to_HM(Seconds,%a)
Description(8,1)="Elapsed time: "+%a

Delete Current Seconds 
Delete T N DF
Delete Y eps res
Delete time time_sq 
Delete ARo ARc ARt ARt2 ARr
Delete NL0   NLo NLc NLt NLd  
Delete Detrend  
Delete SSR 
Delete tau

save {%file}
%msg="Finished: "+@date+" at "+@time
logmsg {%msg}    
StatusLine Finished: N={!N}. Elapsed time {%a}

'END OF PROGRAM
