'The program estimates and tests the AR(1) models with asymptotically
'         subsiding trends as well as ordinary (linear) AR(1) without constant and
'         with constant  for every pair of time series in the specified group. 
'         That is, seres y[rs]=y[r]-y[s] are analyzed.

'         Version: 2020.07.1
           
'         Author: Konstantin Gluschenko
'                       glu@nsu.ru
'                       http://nsu-ef.ru/teachers/Gluschenko_KP/
  
'The program stores results obtained in table PAIRWISE_SUMMARY;  
'         matrix PAIRWISE_MATRIX provides details. Its (r, s)-th element is a 4-digit figure;  
'         a digit equaling 1 means that the relevant model is valid for series y[rs]: 
'         XXXX 
'          | | | |_ ARo    
'          | | |__ ARc 
'          | |___ Convergence (one or more of the trend models is valid) 
'          |____ Divergence (one or more of the trend models is valid with "wrong" sign of delta)
'
'ARGUMENTS of the program
'          % 0: name of group with series to be analyzed
'          other arguments (if needed) may follow in any order:
'          [ADF=no|PP=no]  switch off the ADF or Phillips-Perron test
'          [log=no] [exp=no] [fract=no] switch off models with the respective trend (no more than two!)

'          Examples: states ADF=no
'                              all_regions
'                              regions fract=no PP=no 

'Load a workfile with a group of series to be analyzed before loadng the program.

' Set "Maximum errors before halting" in the Run dialog  to a number 10 or more!

'====================== SUBROUTINES ======================

'************ Read distribution of tau-statistic from Excel file  ****************
Subroutine Get_tau(string %Sheet, vector tau, scalar T, scalar Sample_size_used)
RowVector(255) Sample_size=NA

!errors=@errorcount
p.read(t=xls,rect,s={%Sheet},A7)  {%CVfile}
     if !errors<>@errorcount then
          StatusLine Error while reading {%CVfile} or it does not contain a sheet named {%Sheet}
          STOP
     endif
!errors=@errorcount
Sample_size.read(t=xls,rect,s={%Sheet},B6)  {%CVfile} 'Read line of sample sizes
     for !c=255 to 1 step -1
          if Sample_size(!c)<=T or !c=1 then
               exitloop
          endif
     next
     if Sample_size(!c)=NA then
          StatusLine Error(s) in sample sizes specified in {%CVfile}, sheet {%Sheet}
          STOP
     endif
Sample_size_used=Sample_size(!c)

'Compute the letter symbol of column in %CVfile 
!c=!c+1 
%ABC="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
%Column=""
     !f=@floor(!c/26-0.03)
          if !f>0 then
               %Column=%Column+@mid(%ABC,!f,1)
          endif
!s=!c-!f*26
%Column=%Column+@mid(%ABC,!s,1)
%Cell=%Column+"7"

'Read tau-statistic
tau.read(t=xls,rect,s={%Sheet},{%Cell})  {%CVfile}
     for !i=1 to !N_of_p_values
          if p(!i)=NA or tau(!i)=NA then
               !i=!i+6
               StatusLine Error in the tau-statisic data. See cell A!i or {%Column}!i in {%CVfile}, sheet {%Sheet}.
               STOP
          endif
     next
Delete Sample_size
EndSub 'Get_tau

'******************* Generate initial values of coefficients *********************
Subroutine Best_Fit(series y, string %trend, string %Equation, string %Lags, vector coef)
!minSSR=1e20

' 1. Initial approximation of coefs is 0
c=0
Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)

' 2. Initial approximation of coefs is c for ARo
c=0
Equation ARo.ls D(y)=c(1)*y(-1)
Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)

' 3. Initial approximation of coefs is c for ARc
c=0
Equation ARc.ls D(y)=c(1)*y(-1)+c(4)
c(2)=-c(4)/c(1)
     if c(2)<20 then
               if %trend="log" then
                    c(2)=exp(-c(2))-1
               endif
           Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
     endif        

' 4. Linear approximation
c=0
Equation ARt.ls D(y)=c(1)*y(-1)+c(4)+c(5)*@trend
c(2)=-(c(1)*c(4)+(c(1)+1)*c(5))/(c(1)*c(1))
c(3)=-c(5)/(c(1)*c(2))
     if %trend="fract" and c(3)*(!T_-1)<1 then
          c(3)=-c(3)
          Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
     endif
     if %trend<>"fract" and c(3)*(!T_-1)<20 then
               if %trend<>"log" or (%trend="log" and c(2)>-1 and c(2)*exp(c(3)*(!T_-1))>-1) then 
                    Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
               endif
     endif

' 5. Semi-linear approximation: exp(x) -> 1 + x
     if %trend="log" then
          c=0
          !errors=@errorcount
          Equation ARt.ls(M=1000,C=1e-05) D(y)=c(1)*y(-1)+log(1+c(2)*(1+c(3)*@trend))-(c(1)+1)*log(1+c(2)*(1+c(3)*(@trend-1)))
               if !errors=@errorcount and c(2)>-1 and c(3)*(!T_-1)<20 then
                         if c(2)*exp(c(3)*(!T_-1))>-1 then    
                              Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
                         endif
               endif
      endif

' 6. Semi-linear approximation: ln(1+ x) -> x, simplified exp trend
     if %trend<>"fract" then
          c=0
          !errors=@errorcount
          Equation ARt.ls(M=1000,C=1e-05) D(y)=c(1)*y(-1)+c(2)*exp(c(3)*@trend)*(1-(c(1)+1)*exp(-c(3)))
          c(2)=c(2)/(1-(c(1)+1)*exp(-c(3)))
               if !errors=@errorcount and (%trend="exp" and c(2)<20) or (%trend="log" and c(2)>-1 and c(2)*exp(c(3)*(!T_-1))>-1 and c(3)*(!T_-1)<20) then          
                    Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
               endif
     endif

' 7. Initial approximation is computed thru detrending
c=0
!errors=@errorcount
     if %trend="log" then
          Equation Detrend_log.ls(M=1000,C=1e-05) y=log(1+c(2)*exp(c(3)*@trend))
     endif
     if %trend="exp" then
          Equation Detrend_exp.ls(M=1000,C=1e-05) y=c(2)*exp(c(3)*@trend)
     endif
     if %trend="fract" then
          Equation Detrend_fract.ls(M=1000,C=1e-05) y=c(2)/(1+c(3)*@trend)
     endif
     if !errors=@errorcount then
          series eps=resid
          Equation ARo_eps.ls D(eps)=c(1)*eps(-1)
          Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
     endif
EndSub 'Best_Fit

'********** Estimate model with some initial values of coefficients ************
Subroutine Estimate(series y, string %Equation, string %Lags, string %trend, vector coef, scalar minSSR)

!errors=@errorcount
Equation NL_.ls(M=1000,C=1e-07) {%Equation}{%Lags}

     'Select and store the best estimate
     if !errors=@errorcount then
          Freeze(Eq) NL_.results
          %conv=@left(Eq(6,1),24)
          !fract_err=(%trend="fract" and c(3)*(!T_-1)<=-1)  
               if @stderrs(1)<>NA AND minSSR>@SSR and %conv<>"Convergence not achieved" and !fract_err=0 then
                    minSSR=@SSR
                         for !i=1 to 5+!lagmax
                              coef(!i)=c(!i)     'lambda
                         next
                endif
          Delete Eq
     endif
EndSub 'Estimate

'***************** Perform ADF and Phillips-Perron tests *******************
Subroutine Unit_Root_Tests(series y, series y_detrended, string %trend, string %Equation, scalar SSR_original, scalar se_lambda, scalar tau_lambda, scalar se_resid, vector tau_statistic, scalar lag_length, scalar ADF_t, scalar ADF_p, scalar PP_t, scalar PP_p)

URoot(ADF, none, info=MSIC, save=ADF) y_detrended
lag_length=ADF(2,1)  'optimal number of lags
%Lags=""
     if !lag_length<>NA then
               for !lag=1 to !lag_length
                    %Lags=%Lags+"+c("+@str(!lag+5)+")*D(y("+@str(-!lag)+"))"
               next '!lag
          Call  Best_Fit(y,  %trend, %Equation,%Lags, coefNL)
               for !i=1 to 5+!lag_length
                    c(!i)=coefNL(!i) 
               next '!i
          Equation ADFlag.ls(M=1000,C=1e-07) {%Equation}{%Lags}
               if @stderrs(1)<>NA then  
                    ADF_t=@tstats(1)     'tau ADF
                    '------------------- Compute the Phillips-Perron statistic
                    !betas=1
                         for !i=1 to !lag_length
                              !betas=!betas-c(!i+5)
                         next
                    !g0=SSR_original/(!T_-1)
                    !f0=(SSR_original/(!T_-1))/(!betas*!betas) 
                    PP_t=tau_lambda*sqr(!g0/!f0)-(!T_-1)*(!f0-!g0)*se_lambda/(2*sqr(!f0)*se_resid)

                    '-------------------- Find p-values 
                    PP_p=1
                    ADF_p=1
                         for !i=1 to 1001
                                   if PP_t<=tau_statistic(!i) and PP_p=1 then
                                        PP_p=p(!i) 
                                    endif
                                    if ADF_t<=tau_statistic(!i) and ADF_p=1 then
                                         ADF_p=p(!i) 
                                    endif
                                    if PP_p<1 and ADF_p<1 then
                                         exitloop 
                                    endif
                         next '!i
                    delete ADF
                    delete ADFlag
               else
                    ADF_t=NA
                    ADF_p=NA
                    PP_t=NA
                    PP_p=NA
               endif '@tstats(1)<>NA 
     endif '!lag_length<>NA
EndSub 'Unit_Root_Tests

'====================== PROGRAM BODY ====================

'********************* PREPARATORY PROCEDURES ********************
StatusLine
%Time="Started: "+@datestr(@now, "dd/MM/yyyy HH:mi")+"; elapsed time "
tic        'Reset timer

'Check program arguments     
%0=@upper(%0)               'Convert to capital letters
     if %0="" or @isobject(%0)=0 then
          StatusLine There is no group %0 in your workfile!
          STOP
     endif
     if %4<>""  then
          StatusLine Supefluous argument(s): ... {%4} {%5} {%6} {%7}
          STOP
     endif
!ADF_on=1
!PP_on=1
%log="yes"
%exp="yes"
%fract="yes"
     for %argument 1 2 3
          %arg=%{%argument}
               if %arg="" then
                    exitloop
               endif
               if %arg<>"ADF=no" and %arg<>"PP=no" and %arg<>"log=no" and %arg<>"exp=no" and %arg<>"fract=no"  then
                    StatusLine Invalid argument: %arg
                    STOP
               endif
               if %arg="ADF=no" then
                    !ADF_on=0
               endif
               if %arg="PP=no" then
                    !PP_on=0
               endif
               if %arg="log=no" then
                    %log="no"
               endif
               if %arg="exp=no" then
                    %exp="no"
               endif
               if %arg="fract=no" then
                    %fract="no"
               endif
     next
     if !ADF_on + !PP_on=0 then
          StatusLine Invalid arguments: "ADF=no" and "PP=no". Only one test may be switched off!
          STOP
     endif 
     if %log="no" and %exp="no" and %fract="no" then
          StatusLine Invalid arguments: "log=no", "exp=no", and "fract=no". No more than two models may be switched off!
          STOP
     endif

!N_={%0}.@count               'Number of locations
!T_={%0}.@maxobs           'Sample size
     if @IsObject("PAIRWISE_MATRIX")=1 then
          Delete PAIRWISE_MATRIX
     endif
Matrix(!N_,!N_) PAIRWISE_MATRIX=0

!lagmax=@floor(12*((!T_-1)/100)^0.25)    'maximum number of lags
vector(5+!lagmax) coefNL
Series Yrs
Series Y_detrended
!N_of_p_values=1001
!Significance_level=0.1
Table(3,1) Tau_Status
!Significance_level=0.1
Equation Log_exp_trend
Equation Exp_trend
Equation  Fract_trend

'****************** ESTIMATING AND TESTING: LOG-EXPONENTIAL TREND
if %log<>"no" then 
'--------------- Read tau-statistic
%CVfile="TauStatistics-NonlinearTrends.xls"
!sample_size_used=NA
Vector(!N_of_p_values) p=NA  
Vector(!N_of_p_values) tau_log=NA
Call Get_tau("Log-exp", tau_log, !T_, !sample_size_used)
Tau_Status(1,1)="Log-exponential trend: tau-statistic for T="+@str(!T_)
     if !sample_size_used<>!T_ then
          Tau_Status(1,1)="Log-exponential trend: No tau-statistic for T="+@str(!T_)+" in "+%CVfile+". This for T="+@str(!sample_size_used)+" is used instead."
     endif

'--------------- Estimate
%Eq_log_trend="D(y)=c(1)*y(-1)+log(1+c(2)*exp(c(3)*@trend))-(c(1)+1)*log(1+c(2)*exp(c(3)*(@trend -1)))"

     for !r=1 to !N_
     for !s=!r+1 to !N_
          StatusLine log trend: r=!r, s=!s
          Series Yrs={%0}(!s)-{%0}(!r)
          for !repeat =1 to 2
          coefNL=NA
          !PP_t=NA
          !PPp=NA
          !ADF_t=NA
          !ADFp=NA
          !lag_length=NA
          Call Best_Fit(Yrs, "log", %Eq_log_trend,  "", coefNL)
          c(1)=coefNL(1)
          c(2)=coefNL(2)
          c(3)=coefNL(3)
               if c(1)<>NA then
                    Log_exp_trend.ls(M=1000,C=1e-07) D(Yrs)=c(1)*Yrs(-1)+log(1+c(2)*exp(c(3)*@trend))-(c(1)+1)*log(1+c(2)*exp(c(3)*(@trend -1)))
               endif
               if c(1)<>NA and @stderrs(1)<>NA then
                    !p_gamma=@tdist(@tstats(2), @regobs-@ncoef)
                    !p_delta=@tdist(@tstats(3), @regobs-@ncoef)
                          if !p_gamma<=!Significance_level and !p_delta<=!Significance_level then ' Perform unit root test
                                   if c(3)<0 then
                                        !log_trend=100
                                   else
                                        !log_trend=1000
                                    endif
                              Y_detrended=Yrs - log(1+c(2)*exp(c(3)*@trend))
                              call Unit_Root_Tests(Yrs, Y_detrended, "log", %Eq_log_trend, Log_exp_trend.@SSR, Log_exp_trend.@stderrs(1), Log_exp_trend.@tstats(1), Log_exp_trend.@se, tau_log, !lag_length, !ADF_t, !ADFp, !PP_t, !PPp)
                                   if !ADFp*!ADF_on<=!Significance_level and !PPp*!PP_on<=!Significance_level then
                                        PAIRWISE_MATRIX(!r,!s)=!log_trend
                                        PAIRWISE_MATRIX(!s,!r)=!log_trend
                                        !repeat=10
                                   endif
                         endif 'p_gamma<=!Significance_level...                         
               endif '@stderrs(1)<>NA
          Yrs=-Yrs
     next !repeat
     next '!r 
     next '!s
delete tau_log
endif '%log<>"no" 

'****************** ESTIMATING AND TESTING: EXPONENTIAL TREND
if %exp<>"no" then 
'--------------- Read tau-statistic
%CVfile="TauStatistics-NonlinearTrends.xls"
!sample_size_used=NA
Vector(!N_of_p_values) p=NA  
Vector(!N_of_p_values) tau_exp=NA
Call Get_tau("Exp", tau_exp, !T_, !sample_size_used)
Tau_Status(2,1)="Exponential trend: tau-statistic for T="+@str(!T_)
     if !sample_size_used<>!T_ then
          Tau_Status(2,1)="Exponential trend: No tau-statistic for T="+@str(!T_)+" in "+%CVfile+". This for T="+@str(!sample_size_used)+" is used instead."
     endif

'--------------- Estimate
%Eq_exp_trend="D(y)=c(1)*y(-1)+c(2)*exp(c(3)*@trend)-(c(1)+1)*c(2)*exp(c(3)*(@trend -1))"

     for !r=1 to !N_
     for !s=!r+1 to !N_
          StatusLine exp trend: r=!r, s=!s
          Series Yrs={%0}(!s)-{%0}(!r)
          coefNL=NA
          !PP_t=NA
          !PPp=NA
          !ADF_t=NA
          !ADFp=NA
          !lag_length=NA
          Call Best_Fit(Yrs, "exp", %Eq_exp_trend,  "", coefNL)
          c(1)=coefNL(1)
          c(2)=coefNL(2)
          c(3)=coefNL(3)
               if c(1)<>NA then
                    Exp_trend.ls(M=1000,C=1e-07) D(Yrs)=c(1)*Yrs(-1)+c(2)*exp(c(3)*@trend)-(c(1)+1)*c(2)*exp(c(3)*(@trend -1))
               endif
               if  c(1)<>NA and @stderrs(1)<>NA then
                    !p_gamma=@tdist(@tstats(2), @regobs-@ncoef)
                    !p_delta=@tdist(@tstats(3), @regobs-@ncoef)
                          if !p_gamma<=!Significance_level and !p_delta<=!Significance_level then ' Perform unit root test
                                   if c(3)<0 then
                                        !exp_trend=100
                                   else
                                        !exp_trend=1000
                                    endif
                              Y_detrended=Yrs - c(2)*exp(c(3)*@trend)
                              call Unit_Root_Tests(Yrs, Y_detrended, "exp", %Eq_exp_trend, Exp_trend.@SSR, Exp_trend.@stderrs(1), Exp_trend.@tstats(1), Exp_trend.@se, tau_exp, !lag_length, !ADF_t, !ADFp, !PP_t, !PPp)
                                   if !ADFp*!ADF_on<=!Significance_level and !PPp*!PP_on<=!Significance_level then
                                        PAIRWISE_MATRIX(!r,!s)=!exp_trend
                                        PAIRWISE_MATRIX(!s,!r)=!exp_trend
                                   endif
                         endif 'p_gamma<=!Significance_level...                         
               endif '@stderrs(1)<>NA
     next '!r 
     next '!s
delete tau_exp
endif '%exp<>"no"

'****************** ESTIMATING AND TESTING: FRACTIONAL TREND
if %fract<>"no" then 
'--------------- Read tau-statistic
%CVfile="TauStatistics-NonlinearTrends.xls"
!sample_size_used=NA
Vector(!N_of_p_values) p=NA
Vector(!N_of_p_values) tau_fract=NA
Call Get_tau("Fract", tau_fract, !T_, !sample_size_used)
Tau_Status(3,1)="Fractional trend: tau-statistic for T="+@str(!T_)
     if !sample_size_used<>!T_ then
          Tau_Status(3,1)="Fractional trend: No tau-statistic for T="+@str(!T_)+" in "+%CVfile+". This for T="+@str(!sample_size_used)+" is used instead."
     endif

'--------------- Estimate
%Eq_fract_trend="D(y)=c(1)*y(-1)+c(2)/(1+c(3)*@trend)-(c(1)+1)*c(2)/(1+c(3)*(@trend-1))"

     for !r=1 to !N_
     for !s=!r+1 to !N_
          StatusLine fract trend: r=!r, s=!s
          Series Yrs={%0}(!s)-{%0}(!r)
          coefNL=NA
          !PP_t=NA
          !PPp=NA
          !ADF_t=NA
          !ADFp=NA
          !lag_length=NA
          Call Best_Fit(Yrs, "fract", %Eq_fract_trend,  "", coefNL)
          c(1)=coefNL(1)
          c(2)=coefNL(2)
          c(3)=coefNL(3)
               if c(1)<>NA then
                    Fract_trend.ls(M=1000,C=1e-07) D(Yrs)=c(1)*Yrs(-1)+c(2)/(1+c(3)*@trend)-(c(1)+1)*c(2)/(1+c(3)*(@trend-1))
               endif
               if c(1)<>NA and @stderrs(1)<>NA then
                    !p_gamma=@tdist(@tstats(2), @regobs-@ncoef)
                    !p_delta=@tdist(@tstats(3), @regobs-@ncoef)
                          if !p_gamma<=!Significance_level and !p_delta<=!Significance_level then ' Perform unit root test
                                   if c(3)>0 then
                                        !fract_trend=100
                                   else
                                        !fract_trend=1000
                                    endif
                              Y_detrended=Yrs - c(2)/(1+c(3)*@trend)
                              call Unit_Root_Tests(Yrs, Y_detrended, "fract", %Eq_fract_trend, Fract_trend.@SSR, Fract_trend.@stderrs(1), Fract_trend.@tstats(1), Fract_trend.@se, tau_fract, !lag_length, !ADF_t, !ADFp, !PP_t, !PPp)
                                   if !ADFp*!ADF_on<=!Significance_level and !PPp*!PP_on<=!Significance_level then
                                        PAIRWISE_MATRIX(!r,!s)=!fract_trend
                                        PAIRWISE_MATRIX(!s,!r)=!fract_trend
                                   endif
                         endif 'p_gamma<=!Significance_level...                         
              endif '@stderrs(1)<>NA
     next '!r 
     next '!s
delete tau_fract
endif '%fract<>"no"

'**************** ANALIZING LINEAR AR(1) WITH AND WITH NO CONST **********

%AR="ARdt"
' ATTENTION! This is the value of parameter 'hac' in commands URoot(PP,...) below.
'Its correct value for this program would be %AR="AR".
'There is a mistake in EViews7: it executes the command with 'hac=ARdt' as if is 'hac=AR'. 
'If this mistake is corrected in a later version of EViews, change this operator to   %AR="AR" 

     for !r=1 to !N_
               for !s=!r+1 to !N_
                    StatusLine AR(1) models: r=!r, s=!s
                    Series Yrs={%0}(!s)-{%0}(!r)

                    'AR(1) with no const
                    URoot(ADF, none,  info=MSIC, save=ADF) Yrs
                    '----  hac=Spectral OLS AR !
                    URoot(PP,  none, info=MSIC, save=PP, hac={%AR}) Yrs
                         if ADF(4,1)*!ADF_on<=!Significance_level and PP(4,1)*!PP_on<=!Significance_level then
                                   Pairwise_Matrix(!r,!s)=Pairwise_Matrix(!r,!s)+1
                                   Pairwise_Matrix(!s,!r)=Pairwise_Matrix(!s,!r)+1                                                   
                         endif

                    'AR(1) with const  
                    Equation ARc.ls D(Yrs)=c(1)*Yrs(-1)+c(2)
                    !p_const =@tdist(@tstats(2), @regobs-@ncoef)
                    URoot(ADF, const,  info=MSIC, save=ADF) Yrs
                    '----  hac=Spectral OLS AR !
                    URoot(PP,   const, info=MSIC, save=PP, hac={%AR}) Yrs
                         if ADF(4,1)*!ADF_on<=!Significance_level and PP(4,1)*!PP_on<=!Significance_level and !p_const<=!Significance_level  then
                                   Pairwise_Matrix(!r,!s)=Pairwise_Matrix(!r,!s)+10
                                   Pairwise_Matrix(!s,!r)=Pairwise_Matrix(!s,!r)+10                                                   
                         endif
               next '!s
     next '!r

'******************************* SUMMARIIZING *********************************
     if @IsObject("Pairwise_Summary")=1 then
          Delete Pairwise_Summary
     endif
Table(!N_+2,32) Pairwise_Summary
Pairwise_Summary.setformat(@all) f.0
Pairwise_Summary.setjust(@all) left
Pairwise_Summary(1,1)="No."
Pairwise_Summary(1,2)="Location name"
     for !type=1 to 2
          Pairwise_Summary(2,(!type-1)*5+3)="ARo"
          Pairwise_Summary(2,(!type-1)*5+4)="ARc"
          Pairwise_Summary(2,(!type-1)*5+5)="Converging"
          Pairwise_Summary(2,(!type-1)*5+6)="Diverging"
          Pairwise_Summary(2,(!type-1)*5+7)="No model"
     next 

 '---------------------- ARo -> ARc -> Converging/Diverging
!type=1
Pairwise_Summary(1,(!type-1)*10+3)="SPECIFIC TO GENERAL"
     for !r=1 to !N_
          Pairwise_Summary(!r+2,1)=!r
          Pairwise_Summary(!r+2,2)={%0}.@seriesname(!r)
          !ARo=0
          !ARc=0
          !Converging=0
          !Diverging=0
               for !s =1 to !N_
                   %rs=@str(PAIRWISE_MATRIX(!r,!s),"f4.0")
                        if @mid(%rs,4,1)="1" then
                             !ARo=!ARo + 1 
                        endif
                        if @mid(%rs,3,2)="10"  then
                             !ARc=!ARc + 1
                        endif
                        if @mid(%rs,2,3)="100" then
                             !Converging=!Converging+1
                        endif
                        if @mid(%rs,1,4)="1000" then 
                              !Diverging=!Diverging+1
                        endif
               next '!s
          Pairwise_Summary(!r+2,(!type-1)*5+3)=!ARo
          Pairwise_Summary(!r+2,(!type-1)*5+4)=!ARc
          Pairwise_Summary(!r+2,(!type-1)*5+5)=!Converging
          Pairwise_Summary(!r+2,(!type-1)*5+6)=!Diverging
          Pairwise_Summary(!r+2,(!type-1)*5+7)=(!N_-1) - (!ARo+!ARc+!Converging+!Diverging)
     next '!r

 '---------------------- Converging/Diverging -> ARc -> ARo  
!type=2
Pairwise_Summary(1,(!type-1)*5+3)="GENERAL TO SPECIFIC "
     for !r=1 to !N_
          !ARo=0
          !ARc=0
          !Converging=0
          !Diverging=0
               for !s =1 to !N_
                   %rs=@str(PAIRWISE_MATRIX(!r,!s),"f4.0")
                        if @mid(%rs,1,1)="1" then
                             !Diverging=!Diverging+1
                        endif
                        if @mid(%rs,2,1)="1" then
                             !Converging=!Converging+1
                        endif
                        if @mid(%rs,3,1)="1" and @mid(%rs,1,1)<>"1" and @mid(%rs,2,1)<>"1"  then
                             !ARc=!ARc + 1
                        endif
                        if @mid(%rs,4,1)="1" and @mid(%rs,1,1)<>"1" and @mid(%rs,2,1)<>"1" and @mid(%rs,3,1)<>"1" then
                             !ARo=!ARo + 1
                        endif
               next '!s
          Pairwise_Summary(!r+2,(!type-1)*5+3)=!ARo
          Pairwise_Summary(!r+2,(!type-1)*5+4)=!ARc
          Pairwise_Summary(!r+2,(!type-1)*5+5)=!Converging
          Pairwise_Summary(!r+2,(!type-1)*5+6)=!Diverging
          Pairwise_Summary(!r+2,(!type-1)*5+7)=(!N_-1) - (!ARo+!ARc+!Converging+!Diverging)
      next '!r

     if %log="no" then
          Pairwise_Summary(!N_+4,1)="Models with log-exponential trend were not analyzed"
     else
          Pairwise_Summary(!N_+4,1)=Tau_Status(1,1)
     endif
     if %exp="no" then
          Pairwise_Summary(!N_+5,1)="Models with exponential trend were not analyzed"
     else
          Pairwise_Summary(!N_+5,1)=Tau_Status(2,1)
     endif
     if %log="no" then
          Pairwise_Summary(!N_+6,1)="Models with fractional trend were not analyzed"
     else
          Pairwise_Summary(!N_+6,1)=Tau_Status(3,1)
     endif
!i=0
     if !ADF_on=0 then
          Pairwise_Summary(!N_+7+!i,1)="The ADF test was not applied"
          !i=!i+1
     endif
     if !PP_on=0 then
          Pairwise_Summary(!N_+7+!i,1)="The Phillips-Perron test was not applied"
         !i=!i+1
     endif
Pairwise_Summary(!N_+7+!i,1)="Critical significance level for unit root tests and coefficients is "+@str(!Significance_level)

Delete Tau_Status
delete coefNL
delete Yrs Y_detrended
delete eps
delete ARo ARc Art ARo_eps
delete Detrend*
delete NL_ 
delete p
delete Log_exp_trend Exp_trend Fract_trend
delete ADF PP

!Elapsed=@round(@toc/60)
%Time=%Time+@str(!Elapsed)+" min." 
Pairwise_Summary(!N_+3,1)=%Time
StatusLine %Time

'END of program
