'The program estimates AR(1) models with asymtotically decayng trends and tests them for unit root through the Phillips-Perron test and ADF test.
'The trends are:
'            log-exponential: ln(1 + gamma*exp(delta*t)), 
'            exponential:        gamma*exp(delta*t), 
'            and fractional:    gamma/(1 + delta*t).

'         Version: 2021.07

'         Author: Konstantin Gluschenko
'                       glu@nsu.ru
'                       http://nsu-ef.ru/teachers/Gluschenko_KP/
 
'The program uses one or two arguments; both are series names.
'If one argument is specified, the program analyzes the respective time series;
'if two arguments are specified, the difference of these series is analyzed.
     'Examples;
     'AGUASCALIENTES 
     'BAJA_CALIFORNIA_SUR  zzNational   

'Load a workfile with a group of series to be analyzed before loadng the program.

' Set "Maximum errors before halting" in the Run dialog  to a number 10 or more!

'=================== SUBROUTINES ======================

'************ Read distribution of tau-statistic from Excel file  ****************
Subroutine Get_tau(string %Sheet, vector tau, scalar T, scalar Sample_size_used)
RowVector(255) Sample_size=NA

!errors=@errorcount
p.read(t=xls,rect,s={%Sheet},A7)  {%CVfile}
     if !errors<>@errorcount then
          StatusLine Error while reading {%CVfile} or it does not contain a sheet named {%Sheet}
          STOP
     endif
!errors=@errorcount
Sample_size.read(t=xls,rect,s={%Sheet},B6)  {%CVfile} 'Read line of sample sizes
     for !c=255 to 1 step -1
          if Sample_size(!c)<=T or !c=1 then
               exitloop
          endif
     next
     if Sample_size(!c)=NA then
          StatusLine Error(s) in sample sizes specified in {%CVfile}, sheet {%Sheet}
          STOP
     endif
Sample_size_used=Sample_size(!c)

'Compute the letter symbol of column in %CVfile 
!c=!c+1 
%ABC="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
%Column=""
     !f=@floor(!c/26-0.03)
          if !f>0 then
               %Column=%Column+@mid(%ABC,!f,1)
          endif
!s=!c-!f*26
%Column=%Column+@mid(%ABC,!s,1)
%Cell=%Column+"7"

'Read tau-statistic
tau.read(t=xls,rect,s={%Sheet},{%Cell})  {%CVfile}
     for !i=1 to !N_of_p_values
          if p(!i)=NA or tau(!i)=NA then
               !i=!i+6
               StatusLine Error in the tau-statisic data. See cell A!i or {%Column}!i in {%CVfile}, sheet {%Sheet}.
               STOP
          endif
     next
Delete Sample_size
EndSub 'Get_tau

'******************* Generate initial values of coefficients *********************
Subroutine Best_Fit(series y, string %trend, string %Equation, string %Lags, vector coef)
!minSSR=1e20

' 1. Initial approximation of coefs is 0
c=0
Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)

' 2. Initial approximation of coefs is c for ARo
c=0
Equation ARo.ls D(y)=c(1)*y(-1)
Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)

' 3. Initial approximation of coefs is c for ARc
c=0
Equation ARc.ls D(y)=c(1)*y(-1)+c(4)
c(2)=-c(4)/c(1)
     if c(2)<20 then
               if %trend="log" then
                    c(2)=exp(-c(2))-1
               endif
           Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
     endif        

' 4. Linear approximation
c=0
Equation ARt.ls D(y)=c(1)*y(-1)+c(4)+c(5)*@trend
c(2)=-(c(1)*c(4)+(c(1)+1)*c(5))/(c(1)*c(1))
c(3)=-c(5)/(c(1)*c(2))
     if %trend="fract" and c(3)*(!T_-1)<1 then
          c(3)=-c(3)
          Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
     endif
     if %trend<>"fract" and c(3)*(!T_-1)<20 then
               if %trend<>"log" or (%trend="log" and c(2)>-1 and c(2)*exp(c(3)*(!T_-1))>-1) then 
                    Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
               endif
     endif

' 5. Semi-linear approximation: exp(x) -> 1 + x
     if %trend="log" then
          c=0
          !errors=@errorcount
          Equation ARt1.ls(M=1000,C=1e-05) D(y)=c(1)*y(-1)+log(1+c(2)*(1+c(3)*@trend))-(c(1)+1)*log(1+c(2)*(1+c(3)*(@trend-1)))
               if !errors=@errorcount and c(2)>-1 and c(3)*(!T_-1)<20 then
                         if c(2)*exp(c(3)*(!T_-1))>-1 then    
                              Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
                         endif
               endif
      endif

' 6. Semi-linear approximation: ln(1+ x) -> x, simplified exp trend
     if %trend<>"fract" then
          c=0
          !errors=@errorcount
          Equation ARt2.ls(M=1000,C=1e-05) D(y)=c(1)*y(-1)+c(2)*exp(c(3)*@trend)*(1-(c(1)+1)*exp(-c(3)))
          c(2)=c(2)/(1-(c(1)+1)*exp(-c(3)))
               if !errors=@errorcount and (%trend="exp" and c(2)<20) or (%trend="log" and c(2)>-1 and c(2)*exp(c(3)*(!T_-1))>-1 and c(3)*(!T_-1)<20) then          
                    Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
               endif
     endif

' 7. Initial approximation is computed thru detrending
c=0
!errors=@errorcount
     if %trend="log" then
          Equation Detrend_log.ls(M=1000,C=1e-05) y=log(1+c(2)*exp(c(3)*@trend))
     endif
     if %trend="exp" then
          Equation Detrend_exp.ls(M=1000,C=1e-05) y=c(2)*exp(c(3)*@trend)
     endif
     if %trend="fract" then
          Equation Detrend_fract.ls(M=1000,C=1e-05) y=c(2)/(1+c(3)*@trend)
     endif
     if !errors=@errorcount then
          series eps=resid
          Equation ARo_eps.ls D(eps)=c(1)*eps(-1)
          Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
     endif
EndSub 'Best_Fit

'********** Estimate model with some initial values of coefficients ************
Subroutine Estimate(series y, string %Equation, string %Lags, string %trend, vector coef, scalar minSSR)
!errors=@errorcount
Equation NL_.ls(M=1000,C=1e-07) {%Equation}{%Lags}

     'Select and store the best estimate
     if !errors=@errorcount then
          Freeze(Eq) NL_.results
          %conv=@left(Eq(6,1),24)
          !fract_err=(%trend="fract" and c(3)*(!T_-1)<=-1)  
               if @stderrs(1)<>NA AND minSSR>@SSR and %conv<>"Convergence not achieved" and !fract_err=0 then
                    minSSR=@SSR
                         for !i=1 to 5+!lagmax
                              coef(!i)=c(!i)     'lambda
                         next
                endif
          Delete Eq
     endif
EndSub 'Estimate

'***************** Perform ADF and Phillips-Perron tests *******************
Subroutine Unit_Root_Tests(series y, series y_detrended, string %trend, string %Equation, scalar SSR_original, scalar se_lambda, scalar tau_lambda, scalar se_resid, vector tau_statistic, scalar lag_length, scalar ADF_t, scalar ADF_p, scalar PP_t, scalar PP_p)
URoot(ADF, none, info=MSIC, save=ADF) y_detrended
lag_length=ADF(2,1)  'optimal number of lags
%Lags=""
     if !lag_length<>NA then
               for !lag=1 to !lag_length
                    %Lags=%Lags+"+c("+@str(!lag+5)+")*D(y("+@str(-!lag)+"))"
               next '!lag
          Call  Best_Fit(y,  %trend, %Equation,%Lags, coefNL)
               for !i=1 to 5+!lag_length
                    c(!i)=coefNL(!i) 
               next '!i
          Equation ADFlag.ls(M=1000,C=1e-07) {%Equation}{%Lags}
               if @stderrs(1)<>NA then  
                    ADF_t=@tstats(1)     'tau ADF
                    '------------------- Compute the Phillips-Perron statistic
                    !betas=1
                         for !i=1 to !lag_length
                              !betas=!betas-c(!i+5)
                         next
                    !g0=SSR_original/(!T_-1)
                    !f0=(SSR_original/(!T_-1))/(!betas*!betas) 
                    PP_t=tau_lambda*sqr(!g0/!f0)-(!T_-1)*(!f0-!g0)*se_lambda/(2*sqr(!f0)*se_resid)

                    '-------------------- Find p-values 
                    PP_p=1
                    ADF_p=1
                         for !i=1 to 1001
                                   if PP_t<=tau_statistic(!i) and PP_p=1 then
                                        PP_p=p(!i) 
                                    endif
                                    if ADF_t<=tau_statistic(!i) and ADF_p=1 then
                                         ADF_p=p(!i) 
                                    endif
                                    if PP_p<1 and ADF_p<1 then
                                         exitloop 
                                    endif
                         next '!i
                    delete ADF
                    delete ADFlag
               else
                    ADF_t=NA
                    ADF_p=NA
                    PP_t=NA
                    PP_p=NA
               endif '@tstats(1)<>NA 
     endif '!lag_length<>NA
EndSub 'Unit_Root_Tests

'***************** Add results of unit root test to the estimtion table ***************
Subroutine Edit_Results(table Results, scalar PP_t, scalar PP_p, scalar ADF_t,  scalar ADF_p, scalar lag_length, scalar T_for_tau)
     for !i=1 to 30
          if Results(!i,1)="C(1)" then
               !row=!i
           endif
     next 

setcell(Results,!row,1,"C(1) = lambda", "l") 
setcell(Results,!row+1,1,"C(2) = gamma", "l")
setcell(Results,!row+2,1,"C(3) = delta", "l")
!i=!row+11
Results.setfont(!i) +B
setcell(Results,!row+11,1,"UNIT ROOT TESTS", "l")
Results(!row+12,3)="tau-Statistic"
Results(!row+12,4)="p-Value"
Results(!row+12,5)="Bandwidth/lag length"
setline(Results,!row+13)
setcell(Results,!row+14,1,"Phillips-Perron test", "l")
Results(!row+14,3)=PP_t
Results(!row+14,4)=PP_p
Results(!row+14,5)=lag_length
!i=!row+14
Results.setformat(!i, D) f.3
Results.setformat(!i, E) f.0
setcell(Results,!row+15,1,"Augmented Dickey-Fuller test", "l")
Results(!row+15,3)=ADF_t
Results(!row+15,4)=ADF_p
Results(!row+15,5)=lag_length
!i=!row+15
Results.setformat(!i, D) f.3
Results.setformat(!i, E) f.0
setline(Results,!row+16)
     if T_for_tau<>!T_ then
          Results(!row+17)="There is no tau-statistic for T="+@str(!T_)+" in "+%CVfile+". This for T="+@str(T_for_tau)+" is used instead"
     endif
Show Results
EndSub 'Edit_Results

'====================== PROGRAM BODY ====================

'********************** CHECK PROGRAM ARGUMENTS ********************
StatusLine
     if %0="" then
          StatusLine No series name specified!
          STOP
     endif
%0=@upper(%0)               'Convert to capital letters
     if @isobject(%0)=0 then
          StatusLine There is no series %0 in your workfile!
          STOP
     endif
%Series="Series: "+%0
     if %1<>"" then
          %1=@upper(%1)               'Convert to capital letters
               if @isobject(%1)=0 then
                    StatusLine There is no series %1 in your workfile!
                    STOP
               endif
          %0a=%0
          %1a=%1
               if @length(%0a)+@length(%1a)+1>24 then      'truncate name of the new series
                         if @length(%0a)>12 then
                              %0a=@mid(%0,1,12)
                         endif
                    %1a=@mid(%1,1,23 -  @length(%0a))
               endif 
          %pair=%0a+"_"+%1a
          Series {%pair}={%0}-{%1}
          %Series="Series: "+%pair+" = "+%0+" - "+%1 
          %0=%pair      
     endif   '%1<>""

!T_=@obs({%0})           'Sample size

'***************************** ESTMATION *****************************
logmode logmsg
Series Yrs={%0}
Series Y_detrended
!lagmax=@floor(12*((!T_-1)/100)^0.25)    'maximum number of lags
vector(5+!lagmax) coefNL=NA

%CVfile="TauStatistics-NonlinearTrends.xls"
!N_of_p_values=1001
!sample_size_used=NA

'---------------Estimate and test model with log-exponential trend
 ' *** Read tau-statistic 
Vector(!N_of_p_values) p=NA  
Vector(!N_of_p_values) tau_log=NA
Call Get_tau("Log-exp", tau_log, !T_, !sample_size_used)

'  *** Estimate model
%Eq_log_trend="D(y)=c(1)*y(-1)+log(1+c(2)*exp(c(3)*@trend))-(c(1)+1)*log(1+c(2)*exp(c(3)*(@trend -1)))"
coefNL=NA
!PP_t=NA
!PPp=NA
!ADF_t=NA
!ADFp=NA
!lag_length=NA
Call Best_Fit(Yrs, "log", %Eq_log_trend,  "", coefNL)
!estimated=coefNL(1)
c(1)=coefNL(1)
c(2)=coefNL(2)
c(3)=coefNL(3)
     if coefNL(1)<>NA then
          Equation Log_exponential_trend.ls(M=1000,C=1e-07) D({%0})=c(1)*{%0}(-1)+log(1+c(2)*exp(c(3)*@trend))-(c(1)+1)*log(1+c(2)*exp(c(3)*(@trend -1)))
          freeze(mode=overwrite, Log_exponential_trend_) Log_exponential_trend 
     else
         logmsg Estimation of equation LOG_EXPONENTIAL_TREND is impossible 
     endif

'*** Test for unit root
     if coefNL(1)<>NA and @stderrs(1)<>NA then
          Y_detrended=Yrs - log(1+c(2)*exp(c(3)*@trend))
          call Unit_Root_Tests(Yrs, Y_detrended, "log", %Eq_log_trend, Log_exponential_trend.@SSR, Log_exponential_trend.@stderrs(1), Log_exponential_trend.@tstats(1), Log_exponential_trend.@se, tau_log, !lag_length, !ADF_t, !ADFp, !PP_t, !PPp)
     endif 'coefNL(1)<>NA and @stderrs(1)<>NA
     if !estimated<>NA then
          Call Edit_Results(Log_exponential_trend_, !PP_t, !PPp, !ADF_t,  !ADFp, !lag_length, !sample_size_used)
     endif

'---------------Estimate and test model with exponential trend
' *** Read tau-statistic 
Vector(!N_of_p_values) p=NA
Vector(!N_of_p_values) tau_exp=NA
Call Get_tau("Exp", tau_exp, !T_, !sample_size_used)

'*** Estimate model
%Eq_exp_trend="D(y)=c(1)*y(-1)+c(2)*exp(c(3)*@trend)-(c(1)+1)*c(2)*exp(c(3)*(@trend -1))"
coefNL=NA
!PP_t=NA
!PPp=NA
!ADF_t=NA
!ADFp=NA
!lag_length=NA
Call Best_Fit(Yrs, "exp", %Eq_exp_trend,  "", coefNL)
!estimated=coefNL(1)
c(1)=coefNL(1)
c(2)=coefNL(2)
c(3)=coefNL(3)
     if coefNL(1)<>NA then
          Equation Exponential_trend.ls(M=1000,C=1e-07) D({%0})=c(1)*{%0}(-1)+c(2)*exp(c(3)*@trend)-(c(1)+1)*c(2)*exp(c(3)*(@trend -1))
          freeze(mode=overwrite, Exponential_trend_) Exponential_trend     
     else
         logmsg Estimation of equation EXPONENTIAL_TREND is impossible 
     endif

'*** Test for unit root
     if coefNL(1)<>NA and @stderrs(1)<>NA then
          Y_detrended=Yrs - c(2)*exp(c(3)*@trend)
          call Unit_Root_Tests(Yrs, Y_detrended, "exp", %Eq_exp_trend, Exponential_trend.@SSR, Exponential_trend.@stderrs(1), Exponential_trend.@tstats(1), Exponential_trend.@se, tau_exp, !lag_length, !ADF_t, !ADFp, !PP_t, !PPp)
     endif 'coefNL(1)<>NA and @stderrs(1)<>NA
     if !estimated<>NA then
          Call Edit_Results(Exponential_trend_, !PP_t, !PPp, !ADF_t,  !ADFp, !lag_length, !sample_size_used)
     endif

'---------------Estimate and test model with fractional trend
'*** Read tau-statistic
Vector(!N_of_p_values) p=NA
Vector(!N_of_p_values) tau_fract=NA
Call Get_tau("Fract", tau_fract, !T_, !sample_size_used)

'*** Estimate model
%Eq_fract_trend="D(y)=c(1)*y(-1)+c(2)/(1+c(3)*@trend)-(c(1)+1)*c(2)/(1+c(3)*(@trend-1))"
coefNL=NA
!PP_t=NA
!PPp=NA
!ADF_t=NA
!ADFp=NA
!lag_length=NA
Call Best_Fit(Yrs, "fract", %Eq_fract_trend,  "", coefNL)
!estimated=coefNL(1)
c(1)=coefNL(1)
c(2)=coefNL(2)
c(3)=coefNL(3)
     if coefNL(1)<>NA then
          Equation Fractional_trend.ls(M=1000,C=1e-07) D({%0})=c(1)*{%0}(-1)+c(2)/(1+c(3)*@trend)-(c(1)+1)*c(2)/(1+c(3)*(@trend-1))
          freeze(mode=overwrite, Fractional_trend_) Fractional_trend 
     else
         logmsg Estimation of equation FRACTIONAL_TREND is impossible 
     endif

'*** Test for unit root
     if coefNL(1)<>NA and @stderrs(1)<>NA then
          Y_detrended=Yrs - c(2)/(1+c(3)*@trend)
          call Unit_Root_Tests(Yrs, Y_detrended, "fract", %Eq_fract_trend, Fractional_trend.@SSR, Fractional_trend.@stderrs(1), Fractional_trend.@tstats(1), Fractional_trend.@se, tau_fract, !lag_length, !ADF_t, !ADFp, !PP_t, !PPp)
     endif 'coefNL(1)<>NA and @stderrs(1)<>NA
     if !estimated<>NA then
          Call Edit_Results(Fractional_trend_, !PP_t, !PPp, !ADF_t,  !ADFp, !lag_length, !sample_size_used)
     endif

'--------------------- Finishing
delete coefNL
delete Yrs Y_detrended
delete eps
delete ARo ARc Art ARt1 ARt2 ARo_eps
delete Detrend_log Detrend_exp Detrend_fract 
delete NL_ 
delete p
delete tau_log tau_exp tau_fract
'END of program
